
/*  slsnif.h
 *  Copyright (C) 2001 Yan "Warrior" Gurtovoy (ymg@azstarnet.com)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <termios.h>
#include <signal.h>
#include <wait.h>
#include <sys/stat.h>
#include <getopt.h>
#include <time.h>

#define OPTSTR      "bhtl:s:" /* list of short options for getopt_long() */
#define PORT_OUT    "\nHost   --> "
#define PORT_IN     "\nDevice --> "
#define DEFPTRNAME  "/dev/pty??"
#define TOTALBYTES  "Total bytes transmitted:"
#define MEMFAIL     "Memory allocation failed"
#define PTYFAIL     "Failed to open a pty"
#define PORTFAIL    "Failed to open port"
#define LOGFAIL     "Failed to open log file, defaulting to stdout"
#define SELFAIL     "`Select` failed"
#define RPORTFAIL   "Error reading from port"
#define RPTYFAIL    "Error reading from pty"
#define CLOSEFAIL   "Failed to close file - data loss may occur"
#define RAWFAIL     "Failed to switch into raw mode"
#define PIPEFAIL    "Failed to create pipe"
#define FORKFAIL    "Failed to create child process"
#define RPIPEFAIL   "Error reading from pipe"

#define max(x,y)	((x) > (y) ? (x) : (y))

#define BUFFSIZE    32
#define PRFXSIZE    12
#define DEFBAUDRATE B9600

#ifndef FALSE
#define FALSE       0
#endif
#ifndef TRUE
#define TRUE        1
#endif

typedef struct _tty_struct {
    int             ptyfd;
    int             portfd;
    int             logfd;
    int             ptyraw;
    int             portraw;
    int             ptypipefd[2];
    int             portpipefd[2];
    int             dspbytes;
    int             tstamp;
    struct termios  ptystate_orig;
    struct termios  portstate_orig;
    speed_t         baudrate;
} tty_struct;

static tty_struct   tty_data;
static pid_t        pid = -1;

/* baudrates */
typedef struct _spd_struct {
    speed_t speed;
    char    *spdstr;
} spd_struct;

static spd_struct baudrates[] = {
    {B50,         "50"}, {B75,         "75"}, {B110,         "110"},
    {B134,       "134"}, {B150,       "150"}, {B200,         "200"},
    {B300,       "300"}, {B600,       "600"}, {B1200,       "1200"},
    {B1800,     "1800"}, {B2400,     "2400"}, {B4800,       "4800"},
    {B9600,     "9600"}, {B19200,   "19200"}, {B38400,     "38400"},
    {B57600,   "57600"}, {B115200, "115200"}, {0,             NULL}
};
